; !!!: Screen Ruler(s) (40-column)
;         0               1               2
;         0123456789ABCDEF0123456789ABCDEF01234567
;         0         1         2         3
;         0123456789012345678901234567890123456789
;
;-------------------------------------------------------------------------------
;===============================================================================
;*******************************************************************************

;*******************************************************************************
;
;   Apple ][ SPI test driver
;
;   A driver for my SPI implementation, of which both hardware and software are
;   based on work by André Fachat's solution for the CS/A65
;
;   The idea is to implement this later on in an EEPROM
;
;   We start comments on the same line as operand/etc at column #22!
;
;   Created by Xander Maas on 13/05/16.
;   Copyright © 2016 Xander Maas. All rights reserved.
;
;   Original driver code by André Fachat, Copyright © 1989-1997
;
;   Version history:
;   ================
;   0.1 -   Initial version
;
;   DISCLAIMER:
;   ===========
;   **********************************************
;   ***************   W A R N I N G **************
;   **********************************************
;   THIS PROGRAM IS IN EARLY BETA STAGE, USING IT
;   IS AT YOUR OWN RISK
;   THIS SOFTWARE IS PROVIDED "AS IS" AND ANY
;   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
;   NOT LIMITING TO, THE IMPLIED WARRANTIES OF
;   MERCHANTABILITY AND FITNESS FOR A PARTICULAR
;   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL I BE
;   LIABLE FOR AY DIRECT, INDIRECT, INCIDENTAL,
;   SPECIAL, EXEMPLARY, OR CONSEQUETIAL DAMAGES
;   ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
;   OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
;   DATA, OR PROFITS; OR BUSINESS INTERRUPTIONS)
;   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
;   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;   (INCLUDING INCLUDING NEGLIGENCE OR OTHERWISE)
;   ARISING IN ANY WAY OUT OF USE OF THIS
;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
;   OF SUCH DAMAGE.
;
;*******************************************************************************

;===============================================================================
;   !!!: Assembler/Linker directives
;===============================================================================
.segment "EXEHDR"               ; just to keep the linker happy
.segment "STARTUP"              ; just to keep the linker happy
;         0         1         2         3
;         0123456789012345678901234567890123456789
.linecont +                     ; Allow line continuations

.ifpc02
   USE_65C02 = 1
.else
   USE_65C02 = 0
.endif

;===============================================================================
;   !!!: VIA - SD shift under T2 or PHI2
;===============================================================================
VIA_SHIFT   =   1               ; 0 = use PHI2, 1 = use T2
                                ; Our goal is to use PHI2!

;===============================================================================
;   !!!: DEBUG - If set, we want to output some info
;===============================================================================
DEBUG       =   1               ; 1 = enabled, 0 = disabled

;===============================================================================
;   !!!: Equates
;===============================================================================

;-------------------------------------------------------------------------------
;   FIXME: Slot the card is in! Change it accordingly!
;   We will also save this in the zero page,$06 and x16 at $07
;-------------------------------------------------------------------------------
SLOT        =   $7

;-------------------------------------------------------------------------------
;   !!!: VIA Equates
;-------------------------------------------------------------------------------
VIA_DRB     =   0
VIA_DRA     =   1
VIA_DDRB    =   2
VIA_DDRA    =   3
VIA_T1CL    =   4
VIA_T1CH    =   5
VIA_T1LL    =   6
VIA_T1LH    =   7
VIA_T2CL    =   8
VIA_T2CH    =   9
VIA_SR      =   10
VIA_ACR     =   11
VIA_PCR     =   12
VIA_IFR     =   13
VIA_IER     =   14
VIA_DRBX    =   15

;-------------------------------------------------------------------------------
;   !!!: SD Equates
;-------------------------------------------------------------------------------
SD_CLK      =   4       ; Clock counter, for real world applications,
                        ; set to 128 for debugging? (when shorting MOSI/MISO)?
SD_SEL      =   4
SD_INVCLK   =   2
SD_INVDAT   =   1

;-------------------------------------------------------------------------------
; !!!: Zero Page Usage
;-------------------------------------------------------------------------------
SPSLOT      =   $06
SPSLOTx16   =   $07

CH          =   $24
INVFLAG     =   $32
  kNormal   =   $FF
  kFlash    =   $7F
  kInverse  =   $3F

;Nib  0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
;     --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
; 0 | $   $   $   $   $   $   $   $   $   $   $   $   $   $   $   $
; : |
; 6 | C   C   P   P   P   P   P   P   P   P   #   #   #   #   #   #
; 7 | #   #   #   #   #   #   #   #   #   #   #   #   #   #   #   #
; : |
; F | $   $   $   $   $   $   $   $   $   $   $   $   $   $   $   $
;
; C = SDCOMMAND
; P = SDPOINTER
; # = UNUSED
; $ = RESERVED
;
; Let's hope we can use the $60-$7F zero page locations for our driver,
; otherwise we might have to save/restore them before/after use
SDCOMMAND   =   $60     ; Together with $61
SDPOINTER   =   $62     ; All the way to $69


;-------------------------------------------------------------------------------
; !!!: Apple ][ I/O locations
;-------------------------------------------------------------------------------
KBD         =   $C000
STROBE      =   $C010

;-------------------------------------------------------------------------------
; !!!: Apple ][ Slot I/O locations
;-------------------------------------------------------------------------------
MSLOT       =   $7F8
IOBASE      =   $C080

;-------------------------------------------------------------------------------
; !!!: Apple ][ ROM entries
;-------------------------------------------------------------------------------
AppleSoft   =   $E000       ; Not used when booted into ProDOS (??)
TABV        =   $FB5B
INIT        =   $FB2F
SETPWRC     =   $FB6F
IDBYTE      =   $FBB3
BELL1       =   $FBDD
TEXT        =   $FB2F
CLREOP      =   $FC42       ; Clear to end of page
HOME        =   $FC58
CLRLN       =   $FC95       ; Clear line
CLREOL      =   $FC9C       ; Clear from cursor to end of line
ROMWAIT     =   $FCA8
KEYIN       =   $FD0C
CROUT       =   $FD8E
PRBYTE      =   $FDDA
COUT        =   $FDED
SETINV      =   $FE80
SETNORM     =   $FE84
SetKBD      =   $FE89
SetVID      =   $FE93
PRNUM       =   $FE95

;-------------------------------------------------------------------------------
; !!!: ASCII charcaters
;-------------------------------------------------------------------------------
kLeftArrow  =   $08
kDownArrow  =   $0A
kUpArrow    =   $0B
CR          =   $0D
kRightArrow =   $15
kEscape     =   $1B

;===============================================================================
;   !!!: Macros
;===============================================================================
;   !!!: BRA_OR_JMP
;-------------------------------------------------------------------------------
.macro BRA_OR_JMP Destination
   .IF USE_65C02
       bra Destination
   .ELSE
       jmp Destination
   .ENDIF
.endmacro

;-------------------------------------------------------------------------------
;   !!!: INC_A_OR_ADC1
;-------------------------------------------------------------------------------
.macro INC_A_OR_ADC1
   .IF USE_65C02
      inc a
   .ELSE
      clc
      adc #1
   .ENDIF
.endmacro

;-------------------------------------------------------------------------------
;   !!!: DEC_OR_SBC1
;-------------------------------------------------------------------------------
.macro DEC_A_OR_SBC1
   .IF USE_65C02
      dec a
   .ELSE
      sec
      sbc #1
   .ENDIF
.endmacro

;-------------------------------------------------------------------------------
;   !!!: STZ_OR_LDA_STA
;-------------------------------------------------------------------------------
.macro STZ_OR_LDA_STA mem
   .IF USE_65C02
      stz mem
   .ELSE
      lda #0
      sta mem
   .ENDIF
.endmacro

;-------------------------------------------------------------------------------
;   !!!: STZ_OR_STA_ATADataHigh_X (Just for info)
;-------------------------------------------------------------------------------
.macro STZ_OR_STA_ATADataHigh_X
   .IF USE_65C02
      stz ATADataHigh,x         ; Clear high byte data latch
   .ELSE
      lda #0
      sta ATADataHigh,x         ; Clear high byte data latch
   .ENDIF
.endmacro

;-------------------------------------------------------------------------------
;   !!!: PHY_OR_TYA_PHA
;-------------------------------------------------------------------------------
.macro PHY_OR_TYA_PHA
   .IF USE_65C02
      phy
   .ELSE
      tya
      pha                       ; save Y, it holds the $0n sratchpad RAM offset
   .ENDIF
.endmacro

;-------------------------------------------------------------------------------
;   !!!: PLY_OR_PLA_TAY
;-------------------------------------------------------------------------------
.macro PLY_OR_PLA_TAY
   .IF USE_65C02
      ply
   .ELSE
      pla
      tay
   .ENDIF
.endmacro

;-------------------------------------------------------------------------------
;   !!!: PHX_OR_TXA_PHA
;-------------------------------------------------------------------------------
.macro PHX_OR_TXA_PHA
   .IF USE_65C02
      phx
   .ELSE
      txa
      pha                       ; save Y, it holds the $0n sratchpad RAM offset
   .ENDIF
.endmacro

;-------------------------------------------------------------------------------
;   !!!: PLX_OR_PLA_TAX
;-------------------------------------------------------------------------------
.macro PLX_OR_PLA_TAX
   .IF USE_65C02
      plx
   .ELSE
      pla
      tax
   .ENDIF
.endmacro


;-------------------------------------------------------------------------------
;   !!!: ASSERT
;-------------------------------------------------------------------------------
.macro ASSERT condition, string
   .if condition
   .else
      .error string
   .endif
.endmacro

;-------------------------------------------------------------------------------
;   !!!: WARN
;-------------------------------------------------------------------------------
.macro WARN condition, string
   .if condition
   .else
      .warning string
   .endif
.endmacro

;*******************************************************************************
; Start of our driver code, this might be moved somewhere in the $C800-$CFFE
; memory range
;*******************************************************************************
;   !!!: Start of Driver
;===============================================================================
    .org    $2000
BASEADDR    =   *

start:
    ; Prepare the X and Y registers and save the slot number in the zero page
    ; addresses $06 ($0n) and $07 ($n0)
    ;lda #$C0 + CARD            ; We have to determine the offsets for the
    ldy #SLOT                   ; Y register holds $0n (1..7) for scratchpad RAM
    sty SPSLOT                  ; We store it in the zero page for easy access
    ldx #SLOT * 16              ; X register holds $n0 (1..7) for indexed I/O
    stx SPSLOTx16               ; Which we also save in the zero page

    jsr driverDebug
    rts
;-------------------------------------------------------------------------------
;   driverDebug
;
;   use this routine to test the SPI circuit, short MISO and MOSI so we can send
;   a byte and receive the byte we sent
;-------------------------------------------------------------------------------
driverDebug:
    ldx SPSLOTx16               ; Make sure X-register holds the base IO index
                                ; for our card/slot
    ldy SPSLOT                  ; Make sure Y-register holds the index in case
                                ; we are going to use the scratchpad RAM
    jsr initSPI                 ; Initialize the SPI environment

    ldy #0
@loop:
    tya
    ldx SPSLOTx16               ; Make sure the X-register contains $n0
    jsr readByteSPI             ; Send a byte continuous
    sta $3000,y

    ldx #0
@wait:
    nop
    nop
    dex
    bne @wait

    iny
    bne @loop
    rts

;*******************************************************************************
; !!!: Start of SD Card specific routines
;
;       For more info about the SD/MMC control/command codes, see:
;       http://www.retroleum.co.uk/electronics-articles/basic-mmc-card-access/
;
;       This depends on the macro DEBUG. If this is set (==1), we print some
;       additional info to the screen
;       R ==> SD reset initiated
;*******************************************************************************
;   resetSD
;
;-------------------------------------------------------------------------------
resetSD:
    jsr initSPI

.if DEBUG
    lda #'R'                    ; Output
    jsr COUT
.endif

    jsr deselectSPI             ; Send 10 $ff bytes while deselected
    jsr sendResetBytesSPI

    ; Send SD CMD0
    jsr selectSPI
; Flows into the sendSDCommand0 routine
;   |  |  |  |  |  |  |  |  |  |  |  |
;   v  v  v  v  v  v  v  v  v  v  v  v
;-------------------------------------------------------------------------------
;   sendSDCommand0
;
;   Sends the COMMAND0 command to the SD card
;-------------------------------------------------------------------------------
sendSDCommand0:
    jsr selectSPI

@sendCommmand0:
    lda #<cmd0
    ldy #>cmd0
    jsr sendCommandSD

    jsr waitR1SD
    bcs @timeout2

    cmp #$00                    ; Repeat this, until we receive a $01 from the
    beq @sendCommmand0          ; SD card
    cmp #$01                    ; Proceed when we receive a $01
    bne @error                  ; Otherwise it is considered an error!

.if DEBUG
    php
    pha
    lda #'T'
    jsr COUT
    pla
    plp
.endif

    lda #255
    sta @count
    lsr
    sta @count+1                ; Do this 32768 times (??)
@sendAct2:
    dec @count
    bne @ok2
    dec @count+1
    beq @error
@ok2:
    lda #<appcmd41
    ldy #>appcmd41
    jsr sendApplicationCommandSD

    jsr waitR1SD
    bcs @error

    cmp #$00
    beq @sendBsz
    cmp #$01
    beq @sendAct2
    jsr PRBYTE
.if DEBUG
    pha
    lda #'U'
    jsr COUT
    pla
.endif

    lda #$ff
    sta @count
    lsr
    sta @count+1
@sendAcct:
    dec @count
    bne @ok
    dec @count+1
    beq @error
@ok:
    lda #<cmd1
    ldy #>cmd1
    jsr sendCommandSD

.if DEBUG
    jsr debugHeartbeat
.endif

    jsr waitR1SD
@timeout2:
    bcs @timeout

    cmp #$00
    bne @sendAcct

.if DEBUG
    pha
    lda #'U'
    jsr COUT
    pla
.endif

@sendBsz:
    lda #<cmdBsz
    ldy #>cmdBsz
    jsr sendCommandSD

    jsr waitR1SD
    bcs @timeout

    cmp #$00
    bne @sendBsz

    clc
    bcc @desel

@error:
    ldx #'e'
    .byte   $2c
@timeout:
    ldx #'t'
    pha
    txa
    jsr COUT
    pla
    jsr PRBYTE
    sec

@desel:
    php
    lda #$ff
    jsr sendByteSPI
    jsr deselectSPI
    plp

    rts
@count:
    .word   $0000

;-------------------------------------------------------------------------------
;   sendApplicationCommandSD
;
;   Send an Application command to the SD card
;-------------------------------------------------------------------------------
sendApplicationCommandSD:
    sta @temp                   ; Save our pointer, temporarely
    sty @temp+1

    lda #<appCommand            ; Prepare to send an application command
    ldy #>appCommand            ; to the card
    jsr sendCommandSD

    jsr waitR1SD                ; Wait for the R1 response from the card
    jsr PRBYTE                  ; Print the result

    lda @temp                   ; Restore our pointer to the application
    ldy @temp+1                 ; command and send it
    BRA_OR_JMP sendCommandSD

@temp:
    .word   $0000

;-------------------------------------------------------------------------------
;   sendCommandSD
;
;   Send a command to the SD card
;-------------------------------------------------------------------------------
sendCommandSD:
    sta SDCOMMAND               ; Store the pointer to the command we want to
    sty SDCOMMAND+1             ; send
    ldy #0
@loop:
    lda (SDCOMMAND),y
    jsr sendByteSPI
    iny
    cpy $06                     ; Did we send 6 bytes? (complete command)
    bcc @loop                   ; No? repeat!
    lda #$ff
    jsr sendByteSPI

    rts

;-------------------------------------------------------------------------------
;   waitR1SD
;
;   Wait for the SD card R1 response, should normally be $00, e.g. no error
;-------------------------------------------------------------------------------
waitR1SD:
    jsr readByteSPI
    cmp #$80
    rts

;-------------------------------------------------------------------------------
;   sendCmd10SD
;
;   Send cmd10 to the SD card, which reads the SID register
;-------------------------------------------------------------------------------
sendCmd10SD:
    jsr selectSPI

    lda #<cmd10
    ldy #>cmd10
    jsr sendCommandSD

    jsr waitR1SD
    bcs @timeout

@loop:
    jsr readByteSPI             ; Get the data block
    cmp #$ff                    ; Wait for the 'done' token
    beq @loop

    ldy #16
@loop1:
    jsr readByteSPI             ; Get the data
    jsr PRBYTE                  ; And print the result
    dey
    bne @loop1                  ; We expect 16 bytes

    clc
    .byte $24                   ; Which is a BIT, this consumes the next 'sec'
                                ; so we don't set the just cleared carry ;)
                                ; Nifty method ;)
@timeout:
    sec
    php
.if DEBUG
    bcc @noerr                  ; We can skip this when the carry was cleared,
    pha                         ; there isn't any trouble. Otherwise print a 't'
    lda #'t'                    ; to indicate we experienced a timeout
    jsr COUT
    pla
    jsr PRBYTE
@noerr:
.endif
    jsr deselectSPI
    plp
    rts

;-------------------------------------------------------------------------------
;   sendCmd13SD
;
;   Send cmd13 to the SD card, which reads the CD card's status
;
;   We expect response R2
;-------------------------------------------------------------------------------
sendCmd13SD:
    jsr selectSPI

    lda #<cmd13
    ldy #>cmd13
    jsr sendCommandSD
    jsr waitR1SD                ; The following two lines were not in the
    bcs @error                  ; original code, forgotton?
    jsr readByteSPI
    jsr PRBYTE
    jsr readByteSPI
    jsr PRBYTE

    jsr deselectSPI
    clc
    rts
@error:
    sec
    rts

;-------------------------------------------------------------------------------
;   sendCmd58SD
;
;   Send cmd58 to the SD card, which reads the Operation Condition Register
;   (OCR)
;   We expect response R3
;-------------------------------------------------------------------------------
sendCmd58SD:
    jsr selectSPI

    lda #<cmd58
    ldy #>cmd58
    jsr sendCommandSD
    jsr waitR1SD
    bcs @error
    jsr readByteSPI
    jsr PRBYTE
    jsr readByteSPI
    jsr PRBYTE
    jsr readByteSPI
    jsr PRBYTE
    jsr readByteSPI
    jsr PRBYTE

    jsr deselectSPI
    clc                         ; Make sure carry is cleared
    rts
@error:
    sec
    rts


;*******************************************************************************
; !!!: Data storage for the SD card routines
;*******************************************************************************
cmd0:
    .byte   $40, $00, $00, $00, $00, $95
cmd1:
    .byte   $41, $00, $00, $00, $00, $ff
cmd10:
    .byte   $4a, $00, $00, $00, $00, $ff
cmd13:
    .byte   $4d, $00, $00, $00, $00, $ff
cmdBsz:
    .byte   $50, $00, $00, $02, $00, $ff
cmd58:
    .byte   $7a, $00, $00, $00, $00, $ff
appCommand:
    .byte   $77, $00, $00, $00, $00, $ff
appcmd41:
    .byte   $69, $00, $00, $00, $00, $ff

;*******************************************************************************
; !!!: Start of Low Level SPI
;*******************************************************************************
;   initSPI
;
;   input:
;       Expects the X register to contain the slotnumber for indexed addressing
;       Easiest is to load the X register with the value from SPSLOTx16 ($07)
;-------------------------------------------------------------------------------
initSPI:

    ; Deselect any SPI device
    ; Do invert the clock (mode 0)
    lda IOBASE + VIA_DRA,x      ; We do this indexed (X register) as the card
                                ; might be in some other slot
    ora #SD_SEL + SD_INVCLK
    and #255 - SD_INVDAT
    sta IOBASE + VIA_DRA,x      ; Again indexed with Y register

    ; Set the I/O port of the VIA as output
    lda IOBASE + VIA_DDRA,x     ; Indexed addressing with X Register
    ora #SD_SEL + SD_INVDAT + SD_INVCLK
    sta IOBASE + VIA_DDRA,x

    ; Disable shift register interrupts
    lda #%00000100
    sta IOBASE + VIA_IER,x

.if VIA_SHIFT
    ;===========================================================================
    ; Right now we have two choices, either shift out under T2
    ;
    ; This uses T2 as shift register timer, as it can be made slower. E.g. if
    ; you have slow level converter(s)
    ;===========================================================================

    ; Set timer 2 to timed
    lda IOBASE + VIA_ACR,x
    and #%11011111
    sta IOBASE + VIA_ACR,x

    ; Set timer 2 (Half-cycle is T2+2)
    ; Set low byte first into latch
    lda #SD_CLK                 ; (20160524) Forgot to use uppercase for SD_CLK
    sta IOBASE + VIA_T2CL,x
    ; High byte store sets counter
    lda #0
    sta IOBASE + VIA_T2CH,x

    ; Set up shift register mode to output
    lda IOBASE + VIA_ACR,x
    and #%11110111
    ora #%00010100
    sta IOBASE + VIA_ACR,x

.else
    ;===========================================================================
    ; Or shift out under Phi2 (which has to be re-created using e.g. a 74xxx74
    ; based on the Apple TIL #494 
    ; (Apple II Hardware: 6522 Versatile Interface Adapter (2/97))
    ;
    ; If the hardware is fast enough, you can shift out under control of PHI2
    ;===========================================================================

    ; Set up shift register mode to output under PHI2 control, which makes bits
    ; go out on half PHI2
    lda IOBASE + VIA_ACR,x
    and #%11111011
    ora #%00011000
    sta IOBASE + VIA_ACR,x

.endif

    ; Write first (dummy) byte, to make sure the last bit is low
    lda #0
    sta IOBASE + VIA_SR,x
    rts

;-------------------------------------------------------------------------------
;   deselectSPI
;
;   input:
;       Expects the X register to contain the slotnumber for indexed addressing
;       Easiest is to load the X register with the value from SPSLOTx16 ($07)
;-------------------------------------------------------------------------------
deselectSPI:
    lda IOBASE + VIA_DRA,x
    ora #SD_SEL
    sta IOBASE + VIA_DRA,x
    jsr readByteSPI
    rts

;-------------------------------------------------------------------------------
;   selectSPI
;
;   input:
;       Expects the X register to contain the slotnumber for indexed addressing
;       Easiest is to load the X register with the value from SPSLOTx16 ($07)
;-------------------------------------------------------------------------------
selectSPI:
    lda IOBASE + VIA_DRA,x
    and #255-SD_SEL
    sta IOBASE + VIA_DRA,x
    rts

;-------------------------------------------------------------------------------
;   sendResetBytesSPI
;
;   Sends a $ff byte and keeping the data line high, which is needed for an MMC
;   card to switch to SPI mode
;
;   input:
;       Expects the X register to contain the slotnumber for indexed addressing
;       Easiest is to load the X register with the value from SPSLOTx16 ($07)
;-------------------------------------------------------------------------------
sendResetBytesSPI:
    ; Invert data so first bit is already high
    inc IOBASE + VIA_DRA,x
    ldy #10                     ; The original code by André used the X-register
@loop:                          ; but we need this for indexed addressing, thus
    lda #$00                    ; we are using the Y register instead
    sta IOBASE + VIA_SR,x

    lda #%00000100              ; Wait to finish
@wait1:
    ;bit IOBASE + VIA_IFR,x     ; This is illegal, so we are going to use other
                                ; zero page address as temporary storage ;)
    lda IOBASE + VIA_IFR,x
    sta testByte
    bit testByte
    beq @wait1
@end:
    sta IOBASE + VIA_IFR,x      ; Clear interrupt(s)

    dey                         ; Next Reset byte
    bne @loop

    dec IOBASE + VIA_DRA,x      ; Reset inverter
    rts

;-------------------------------------------------------------------------------
;   read/sendByteSPI
;
;   Reads a byte, flows into spiSendByte
;
;   input:
;       Expects the X register to contain the slotnumber for indexed addressing
;       Easiest is to load the X register with the value from SPSLOTx16 ($07)
;-------------------------------------------------------------------------------
readByteSPI:
    lda #$ff

; Flows into:
; vvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
sendByteSPI:

    ; mode 0 : make sure the last bit is 0, shifting bit 7 into carry
    asl
    bcs @invert                 ; Seems carry is set, which means bit 7 is/was
                                ; 1

    ; Last bit was 0, nothing to do but send byte
    sta IOBASE + VIA_SR,x

    lda #%00000100              ; Wait to finish

@wait0:                         ; (20160526) Hmmm, somehow this will not work(?)
                                ;            Instead of BIT I am going to use
    and IOBASE + VIA_IFR,x      ;            AND addr,X
                                ;            This might be tricky because of
                                ;            false reads(??)

    ;bit testByte
    beq @wait0
    bne @end

@invert:                        ; (20160522) Forgot to compensate for 
                                ;            indexed I/O
    inc IOBASE + VIA_DRA,x      ; Invert the current bit (which is last bit from
                                ; previous data byte, which we set to zero (0)
    eor #$fe                    ; Compensate for the inversion
    sta IOBASE + VIA_SR,x       ; Send out the data
    lda #%00000100              ; Wait to finish

@wait1:                         ; (20160526) Hmmm, somehow this will not work(?)
                                ;            Instead of BIT I am going to use
    and IOBASE + VIA_IFR,x      ;            AND addr,X
                                ;            This might be tricky because of
                                ;            false reads(??)

    ;bit testByte
    beq @wait1

                                ; (20160522) Forgot to compensate for 
                                ;            indexed I/O
    dec IOBASE + VIA_DRA,x      ; Reset inverter

@end:
    lda #%00000100              ; (20160525) Load the A with %00000100, so
                                ;            we can reset the IFR
    sta IOBASE + VIA_IFR,x      ; Clear interrupt

                                ; (20160522) Forgot to compensate for
                                ;            indexed I/O
    lda IOBASE + VIA_DRB,x      ; Read data, load from external shift register
    rts

;-------------------------------------------------------------------------------
;   TODO: We might have to move this to zero page storage(??)
;-------------------------------------------------------------------------------
testByte:                       ; (20160525) This is the %00000100 we are using
                                ; for the BIT test and the IFR reset
    .byte $04                   ; Where we can store the byte to BIT test
                                ; I am not sure where I should put this when I
                                ; am going to implement this in a ROM. Might
                                ; consider using a ZP address

;*******************************************************************************
; !!!: Some debug routine(s)
;*******************************************************************************
;   debugHeartbeat
;
;   Just prints a '.' to the screen
;-------------------------------------------------------------------------------
debugHeartbeat:
    pha
    lda #'.'
    jsr COUT
    pla
    rts


CODESIZE    =   *-BASEADDR
    .out .concat("Size of code: $", .sprintf("%04x",CODESIZE), " (", .sprintf("%05d",CODESIZE), ")")




